// DirDrawView.cpp : implementation of the CDirDrawView class
//

#include "stdafx.h"
#include "DirDraw.h"

#include "DirDrawDoc.h"
#include "DirDrawView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDirDrawView

IMPLEMENT_DYNCREATE(CDirDrawView, CPrintfView)

BEGIN_MESSAGE_MAP(CDirDrawView, CPrintfView)
	//{{AFX_MSG_MAP(CDirDrawView)
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDirDrawView construction/destruction

CDirDrawView::CDirDrawView()
{
	m_pDD = NULL;
#ifdef USE_HWND
	m_hwndDD = NULL;
#else
	m_pWndDD = NULL;
#endif
	m_pD3D = NULL;
	m_pStage = NULL;
	m_pScene = NULL;
	m_bEnableUpdates = FALSE;
}

CDirDrawView::~CDirDrawView()
{
}

BOOL CDirDrawView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CPrintfView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CDirDrawView drawing

void CDirDrawView::OnDraw(CDC* pDC)
{
	CDirDrawDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CDirDrawView diagnostics

#ifdef _DEBUG
void CDirDrawView::AssertValid() const
{
	CPrintfView::AssertValid();
}

void CDirDrawView::Dump(CDumpContext& dc) const
{
	CPrintfView::Dump(dc);
}

CDirDrawDoc* CDirDrawView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CDirDrawDoc)));
	return (CDirDrawDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDirDrawView message handlers

static int GetBPP(DWORD dwFlags)
{
	if (dwFlags & DDBD_32) {
		return 32;
	} else if (dwFlags & DDBD_24) {
		return 24;
	} else if (dwFlags & DDBD_16) {
		return 16;
	} else if (dwFlags & DDBD_8) {
		return 8;
	} else if (dwFlags & DDBD_4) {
		return 4;
	} else if (dwFlags & DDBD_2) {
		return 2;
	} else if (dwFlags & DDBD_1) {
		return 1;
	}
	return 0;
}

#ifdef USE_HWND
// Demo window window proc
long FAR PASCAL WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{




	return DefWindowProc(hWnd, msg, wParam, lParam);
}
#endif

void CDirDrawView::CleanUp()
{
	m_bEnableUpdates = FALSE;
	if (m_pScene) {
		delete m_pScene;
		m_pScene = NULL;
	}
	if (m_pStage) {
		delete m_pStage;
		m_pStage = NULL;
	}
	if (m_pD3D) {
		delete m_pD3D;
		m_pD3D = NULL;
	}
#ifdef USE_HWND
	if (m_hwndDD) {
		::DestroyWindow(m_hwndDD);
		m_hwndDD = NULL;
	}
#else
	if (m_pWndDD) {
		m_pWndDD->DestroyWindow();
		delete m_pWndDD;
		m_pWndDD = NULL;
	}
#endif
	if (m_pDD) {
		delete m_pDD;
		m_pDD = NULL;
	}
}

void CDirDrawView::OnDestroy() 
{
	CPrintfView::OnDestroy();
	
	CleanUp();
}

void CDirDrawView::OnInitialUpdate() 
{
	CPrintfView::OnInitialUpdate();
	
	Print("Direct Draw demonstration");
	Print("=========================");
	Print("");

	// delete any existing objects we have
	CleanUp();

	// Show the list of installed DD devices
	int iNum = theDDEngine.GetNumDevices();
	if (iNum == 0) {
		Print("No installed Direct Draw devices found");
		return;
	}
	Print("%d Direct Draw devices:", iNum);
	CString strName;
	CString strDesc;
	for (int i = 0; i < iNum; i++) {
		theDDEngine.GetDeviceInfo(i, strName, strDesc);
		Print("  Name: %s", (const char*)strName);
		Print("  Desc: %s", (const char*)strDesc);
	}
	Print("");

	// Create a DD object to play with
	m_pDD = new CDirectDraw;
	BOOL b = m_pDD->Create();
	ASSERT(b);

	// Get the device capabilities
	DDCAPS hw;
	DDCAPS hel;
	m_pDD->GetCaps(&hw, &hel);

	Print("Hardware Caps:");
	Print("  %lu KB of video memory, %lu KB free",
		  hw.dwVidMemTotal/1024, hw.dwVidMemFree/1024);
	Print("  %s 3D Accelerator", (hw.dwCaps & DDCAPS_3D) ? "Has" : "No");
	Print("  %s Blt", (hw.dwCaps & DDCAPS_BLT) ? "Has" : "No");
	Print("  %s Stretch Blt", (hw.dwCaps & DDCAPS_BLTSTRETCH) ? "Has" : "No");
	Print("  %s Overlay", (hw.dwCaps & DDCAPS_OVERLAY) ? "Has" : "No");
	Print("  %s Alpha", (hw.dwCaps & DDCAPS_ALPHA) ? "Has" : "No");
	Print("  %s Certified", (hw.dwCaps2 & DDCAPS2_CERTIFIED) ? "Is" : "Not");
	if (hw.dwZBufferBitDepths) {
		Print("  %d bpp Z buffer", GetBPP(hw.dwZBufferBitDepths));
	} else {
		Print("  Unknown Z buffer bit depth");
	}
	Print("");


	Print("HEL Caps:");
	Print("  %lu KB of video memory, %lu KB free",
		  hel.dwVidMemTotal/1024, hel.dwVidMemFree/1024);
	Print("  %s 3D Accelerator", (hel.dwCaps & DDCAPS_3D) ? "Has" : "No");
	Print("  %s Blt", (hel.dwCaps & DDCAPS_BLT) ? "Has" : "No");
	Print("  %s Stretch Blt", (hel.dwCaps & DDCAPS_BLTSTRETCH) ? "Has" : "No");
	Print("  %s Overlay", (hel.dwCaps & DDCAPS_OVERLAY) ? "Has" : "No");
	Print("  %s Alpha", (hel.dwCaps & DDCAPS_ALPHA) ? "Has" : "No");
	Print("  %s Certified", (hel.dwCaps2 & DDCAPS2_CERTIFIED) ? "Is" : "Not");
	if (hel.dwZBufferBitDepths) {
		Print("  %d bpp Z buffer", GetBPP(hel.dwZBufferBitDepths));
	} else {
		Print("  Unknown Z buffer bit depth");
	}
	Print("");

	// Get the full-screen display modes
	int iModes = m_pDD->GetNumModes();
	Print("%d full-screen display modes:", iModes);
	for (i = 0; i < iModes; i++) {
		DDSURFACEDESC dds;
		m_pDD->GetModeInfo(i, &dds);
		Print("  %lu x %lu x %lu",
			  dds.dwWidth,
			  dds.dwHeight,
			  dds.ddpfPixelFormat.dwRGBBitCount);
	}
	Print("");

	int cx = 320;
	int cy = 240;
	m_rcWnd.top = 0;
	m_rcWnd.left = 0;
	m_rcWnd.bottom = m_rcWnd.top + cy;
	m_rcWnd.right = m_rcWnd.left + cx;

#ifdef USE_HWND
	// Register our demo window class
	WNDCLASS wc;
	memset(&wc, 0, sizeof(wc));
	wc.style = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc = WndProc;
	wc.hInstance = AfxGetInstanceHandle();
	wc.hCursor = ::LoadCursor(NULL, IDC_ARROW);
	wc.lpszClassName = "DDHwndClass";
	wc.hbrBackground = (HBRUSH)::GetStockObject(GRAY_BRUSH);
	::RegisterClass(&wc);

	// Create the demo window
	m_hwndDD = ::CreateWindowEx(WS_EX_APPWINDOW,
								 "DDHwndClass",
								 "HWND Example",
								 WS_VISIBLE | WS_POPUP | WS_CAPTION,
								 m_rcWnd.left, m_rcWnd.top,
								 cx, cy,
								 NULL,
								 NULL,
								 AfxGetInstanceHandle(),
								 NULL);
	ASSERT(m_hwndDD);
	::UpdateWindow(m_hwndDD);

	// Set the mode for the window
	b = m_pDD->SetWindowedMode(m_hwndDD, cx, cy);
	ASSERT(b);
#else
	CString strClass =
		AfxRegisterWndClass(CS_HREDRAW | CS_VREDRAW,
							::LoadCursor(NULL, IDC_ARROW),
							(HBRUSH)::GetStockObject(GRAY_BRUSH));
	m_pWndDD = new CWnd;
	m_pWndDD->CreateEx(0,
					   strClass,
					   "CWnd Example",
					   WS_VISIBLE | WS_POPUP | WS_CAPTION,
					   m_rcWnd.left, m_rcWnd.top,
					   cx, cy,
					   NULL,
					   NULL);
	m_pWndDD->UpdateWindow();

	// Set the mode for the window
	b = m_pDD->SetWindowedMode(m_pWndDD->GetSafeHwnd(), cx, cy);
	ASSERT(b);
#endif

	// Create the Direct3D object
	m_pD3D = new CDirect3D;
	b = m_pD3D->Create(m_pDD);
	ASSERT(b);

	// Set the color model we want
	b = m_pD3D->SetMode(D3DCOLOR_RGB);
	ASSERT(b);
	
	// Create a stage
	m_pStage = new C3dStage;
	b = m_pStage->Create(m_pD3D);

	// Create an initial scene
	m_pScene = new C3dScene;
	m_pScene->Create();
	m_pStage->SetScene(m_pScene);

	// Set up the lighting
    C3dDirLight dl;
    dl.Create(0.8, 0.8, 0.8);
    m_pScene->AddChild(&dl);
    dl.SetPosition(-2, 2, -5);
    dl.SetDirection(1, -1, 1);
    m_pScene->SetAmbientLight(0.4, 0.4, 0.4);

	// Create a shape to add
	C3dShape sh1;
	sh1.CreateSphere(1);
	m_pScene->AddChild(&sh1);
	C3dShape sh2;
	sh2.CreateSphere(0.3);
	sh2.SetColor(0, 0, 1);
	sh1.AddChild(&sh2);
	sh2.SetPosition(0, 0, -2);
	C3dShape sh3;
	sh3.CreateSphere(0.15);
	sh3.SetColor(1, 0, 0);
	sh1.AddChild(&sh3);
	sh3.SetPosition(0, 0, 5);
	sh1.SetRotation(1, 1, 0, 0.015);

	// Render the initial scene and show it
	DWORD dwStart = timeGetTime();
	m_pStage->Render();
	DWORD dwRender = timeGetTime() - dwStart;

	Print("Render: %lu ms", dwRender);

	// Blt the back buffer to the front buffer so we
	// can see what we rendered
	RECT rcFrom;
	rcFrom.left = 0;
	rcFrom.top = 0;
	rcFrom.right = cx;
	rcFrom.bottom = cy;
	dwStart = timeGetTime();
	b = m_pDD->GetFrontBuffer()->Blt(&m_rcWnd,
								 m_pDD->GetBackBuffer(),
								 &rcFrom,
								 DDBLT_WAIT,
								 NULL);
	ASSERT(b);
	DWORD dwBlt = timeGetTime() - dwStart;
	Print("Blt: %lu ms", dwBlt);

	// Enable the idle-time redraw
	m_bEnableUpdates = TRUE;


	Print("");
}


BOOL CDirDrawView::OnIdle()
{
	if (!m_bEnableUpdates) return FALSE;

	// update the scene
    m_pScene->Move(1);
    m_pStage->Render();
	BOOL b;
	// Blt the back buffer to the front buffer so we
	// can see what we rendered
	RECT rcFrom;
	rcFrom.left = 0;
	rcFrom.top = 0;
	rcFrom.right = m_rcWnd.right - m_rcWnd.left;
	rcFrom.bottom = m_rcWnd.bottom - m_rcWnd.top;
	b = m_pDD->GetFrontBuffer()->Blt(&m_rcWnd,
								 m_pDD->GetBackBuffer(),
								 &rcFrom,
								 DDBLT_WAIT,
								 NULL);

	return TRUE;
}
